/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "MysticRealms.h"



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

MysticRealms::MysticRealms() {
    gameState = VD_GAMEWORLD;
    gameRunning = TRUE;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

MysticRealms::~MysticRealms() {
};



/* **********************************************************************************************************************
   Run
   ---
   Starts running the Mystic Realms game.
   #param  instance  Instance of the application.
   **********************************************************************************************************************/

void MysticRealms::Run(HINSTANCE instance) {
    VDWindow window;
    VDDirect3D graphics;
    VDInput input;
    VDWorld world;
    VDEntity player;
    DWORD totalMillisecondsElapsed;
    MSG message;
    int width, height;

    srand((unsigned int) time(NULL));
    player.SetCurrentLocation(8, 8);
    width = 800;
    height = 600;

    // create the application window
    window.CreateMyWindow(instance, "Mystic Realms", width, height);

    // initialize the graphics API
    graphics.InitializeDirect3D(window.GetWindow());

    // initialize input
    input.InitializeInput(instance);
    input.InitializeKeyboard(window.GetWindow());

    // load world graphics
    world.LoadWorld("test.vd");
    for (int x = 0; x < world.GetWorldSizeX(); x++) {
        for (int y = 0; y < world.GetWorldSizeY(); y++) {
            graphics.AddSprite(world.GetTileImageFileLocation(x, y),
                               (float) x * 32.0f,
                               (float) y * 32.0f,
                               32,
                               32,
                               FALSE,
                               FALSE);
        };
    };

    graphics.AddSprite("../MysticRealms/images/characters/dg_classm32_10.png", (float) width / 2, (float) height / 2, 32, 32, TRUE, FALSE);

    /*const short framesPerSecond = 50;
    const short skipFrames = 1000 / framesPerSecond;
    const short maxFrameskip = 10;*/

    totalMillisecondsElapsed = GetTickCount();

    // game loop
    while (gameRunning) {
        // dispatch incoming sent messages, check the thread message queue, retrieve message
        if (PeekMessage(&message, window.GetWindow(), 0, 0, PM_REMOVE)) {
            TranslateMessage(&message);
            DispatchMessage(&message);
        } else {
            // 1. check for user input
            // 2. run AI
            // 3. move enemies
            // 4. resolve collisions
            // 5. draw graphics
            // 6. play sounds

            // gather input
            input.RetrieveLostInput();
            input.UpdateInput();

            // Game State:  World
            if (gameState == VD_GAMEWORLD && graphics.IsMoving() == FALSE) {
                short choice = VD_NONE;

                // see if any keys were pressed
                if (input.GetKeyboardInput()[DIK_ESCAPE] & 0x80) {
                    gameRunning = FALSE;
                } else if (input.GetKeyboardInput()[DIK_LEFT] & 0x80) {
                    choice = VD_LEFT;
                } else if (input.GetKeyboardInput()[DIK_RIGHT] & 0x80) {
                    choice = VD_RIGHT;
                } else if (input.GetKeyboardInput()[DIK_UP] & 0x80) {
                    choice = VD_UP;
                } else if (input.GetKeyboardInput()[DIK_DOWN] & 0x80) {
                    choice = VD_DOWN;
                };

                // user decided to move
                if (choice >= 1 && choice <= 4) {
                    // see if the new tile is available
                    int checkNewLocation = world.CheckNewLocation(choice, player.GetCurrentLocationX(), player.GetCurrentLocationY());
                    switch (checkNewLocation) {
                        // null
                        case 0:
                            break;
                        // open
                        case 1:
                            // update the player location
                            player.Move(choice);

                            // let the graphics know we're moving
                            graphics.MovingPlayer(choice);
                            break;
                        // impassable
                        case 2:
                            break;
                        // new area
                        case 3:
                            break;
                    };
                };
            };

            // draw graphics
            RECT drawingArea;
            drawingArea.left = 0;
            drawingArea.top = 0;
            drawingArea.right = width;
            drawingArea.bottom = height;
            graphics.Render(&drawingArea, width, height, player.GetCurrentLocationX(), player.GetCurrentLocationY());
        };

        while ((GetTickCount() - totalMillisecondsElapsed) < 250);
    };
};