/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDNPCEditor.h"



/* **********************************************************************************************************************
   Global Function Prototypes
   **********************************************************************************************************************/

LRESULT CALLBACK WindowProcedure(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam);



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDNPCEditor::VDNPCEditor() {
    window = NULL;
    npcNameStatic = NULL;
    npcNameEditBox = NULL;
    loadSpriteButton = NULL;
    npcSprite = NULL;
    (void) SecureZeroMemory(&npcSpriteDimensions, sizeof(npcSpriteDimensions));
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDNPCEditor::~VDNPCEditor() {
    if (npcSprite != NULL) {
        delete npcSprite;
        npcSprite = NULL;
    };
};



/* **********************************************************************************************************************
   CreateNPCEditor
   ---------------
   Creates a new NPC Editor window.
   #param  instance   The instance of the application.
   #param  className  The name of the window class.
   #return  Handle to the window.
   **********************************************************************************************************************/

HWND VDNPCEditor::CreateNPCEditor(HINSTANCE instance, LPCSTR className) {
    window = CreateWindowEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW,      // extended window styles
                            className,                             // class name
                            "NPC Editor",                          // window name
                            WS_CAPTION | WS_SYSMENU | WS_VISIBLE,  // window styles
                            200,                                   // x coordinate (0 = left)
                            200,                                   // y coordinate (0 = top)
                            400,                                   // width
                            200,                                   // height
                            NULL,                                  // parent/owner window
                            NULL,                                  // menu
                            instance,                              // instance
                            NULL);                                 // used for CREATESTRUCT structure
    if (window == NULL) {
        VDError("Failed to create the window.", __FILE__, __LINE__);
    };

    npcNameStatic = CreateWindow("STATIC", "Name:", WS_CHILD | WS_VISIBLE, 10, 10, 50, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_NAME, instance, NULL);
    if (npcNameStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    npcNameEditBox = CreateWindow("EDIT", "", WS_CHILD | WS_VISIBLE, 60, 10, 140, 18, window, (HMENU) VD_NPC_EDITOR_NAME, instance, NULL);
    if (npcNameEditBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    loadSpriteButton = CreateWindow("BUTTON", "Load Sprite", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 210, 10, 90, 20, window, (HMENU) VD_NPC_EDITOR_LOAD_SPRITE, instance, NULL);
    if (loadSpriteButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    npcSprite = NULL;
    npcSpriteDimensions.left = 310;
    npcSpriteDimensions.top = 10;
    npcSpriteDimensions.right = 342;
    npcSpriteDimensions.bottom = 42;

    return window;
};



/* **********************************************************************************************************************
   Accessors and Modifiers
   **********************************************************************************************************************/

HWND VDNPCEditor::GetWindow() {
    return window;
};

HWND VDNPCEditor::GetNPCNameStatic() {
    return npcNameStatic;
};

HWND VDNPCEditor::GetNPCNameEditBox() {
    return npcNameEditBox;
};

HWND VDNPCEditor::GetLoadSpriteButton() {
    return loadSpriteButton;
};

VDSprite* VDNPCEditor::GetNPCSprite() {
    return npcSprite;
};

void VDNPCEditor::SetNPCSprite(VDSprite *newSprite) {
    if (npcSprite != NULL) {
        delete npcSprite;
        npcSprite = NULL;
    };
    npcSprite = newSprite;
};

LPRECT VDNPCEditor::GetNPCSpriteDimensions(LPRECT outRect) {
    outRect->left = npcSpriteDimensions.left;
    outRect->top = npcSpriteDimensions.top;
    outRect->right = npcSpriteDimensions.right;
    outRect->bottom = npcSpriteDimensions.bottom;

    return &npcSpriteDimensions;
};